/*
    Copyright (C) 2010 wpitchoune@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <gconf/gconf-client.h>

#include "config.h"

#define KEY_GRAPH_BACKGROUND_COLOR "/apps/psensor/graph/background_color"
#define DEFAULT_GRAPH_BACKGROUND_COLOR "#e8f4e8f4a8f5"

#define KEY_GRAPH_BACKGROUND_ALPHA  "/apps/psensor/graph/background_alpha"
#define DEFAULT_GRAPH_BACKGROUND_ALPHA "1.0"

#define KEY_GRAPH_FOREGROUND_COLOR  "/apps/psensor/graph/foreground_color"
#define DEFAULT_GRAPH_FOREGROUND_COLOR "#000000000000"

#define KEY_ALPHA_CHANNEL_ENABLED  "/apps/psensor/graph/alpha_channel_enabled"
#define DEFAULT_ALPHA_CHANNEL_ENABLED 0

GConfClient *client;

char *config_get_string(char *key, char *default_value)
{
	char *value = gconf_client_get_string(client,
					      key,
					      NULL);

	if (!value) {
		value = strdup(default_value);

		gconf_client_set_string(client, key, default_value, NULL);
	}

	return value;
}

struct color *config_get_background_color()
{

	char *scolor = config_get_string(KEY_GRAPH_BACKGROUND_COLOR,
					 DEFAULT_GRAPH_BACKGROUND_COLOR);

	struct color *c = string_to_color(scolor);

	free(scolor);

	if (c == NULL)
		return color_new(0xffff, 0xffff, 0xffff);

	return c;
}

struct color *config_get_foreground_color()
{
	char *scolor = config_get_string(KEY_GRAPH_FOREGROUND_COLOR,
					 DEFAULT_GRAPH_FOREGROUND_COLOR);

	struct color *c = string_to_color(scolor);

	free(scolor);

	if (c == NULL)
		return color_new(0x0000, 0x0000, 0x0000);

	return c;
}

int config_is_alpha_channel_enabled()
{
	gboolean b = gconf_client_get_bool(client,
					   KEY_ALPHA_CHANNEL_ENABLED,
					   NULL);

	return b == TRUE;
}

void config_set_alpha_channel_enabled(int enabled)
{
	if (enabled)
		gconf_client_set_bool(client,
				      KEY_ALPHA_CHANNEL_ENABLED, TRUE, NULL);
	else
		gconf_client_set_bool(client,
				      KEY_ALPHA_CHANNEL_ENABLED, FALSE, NULL);
}

double config_get_graph_background_alpha()
{
	double a = gconf_client_get_float(client,
					  KEY_GRAPH_BACKGROUND_ALPHA,
					  NULL);

	if (a == 0)
		gconf_client_set_float(client,
				       KEY_GRAPH_BACKGROUND_ALPHA, 1.0, NULL);
	return a;
}

void config_set_graph_background_alpha(double alpha)
{
	gconf_client_set_float(client, KEY_GRAPH_BACKGROUND_ALPHA, alpha, NULL);
}

void config_set_background_color(struct color *color)
{
	char *scolor = color_to_string(color);

	if (!scolor)
		scolor = strdup(DEFAULT_GRAPH_BACKGROUND_COLOR);

	gconf_client_set_string(client,
				KEY_GRAPH_BACKGROUND_COLOR, scolor, NULL);

	free(scolor);
}

void config_set_foreground_color(struct color *color)
{
	char *scolor = color_to_string(color);

	if (!scolor)
		scolor = strdup(DEFAULT_GRAPH_FOREGROUND_COLOR);

	gconf_client_set_string(client,
				KEY_GRAPH_FOREGROUND_COLOR, scolor, NULL);

	free(scolor);
}

char *config_get_sensor_key(char *sensor_name)
{
	char *escaped_name = gconf_escape_key(sensor_name, -1);
	// /apps/psensor/sensors/[sensor_name]/color
	char *key = malloc(22 + 2 * strlen(escaped_name) + 6 + 1);

	sprintf(key, "/apps/psensor/sensors/%s/color", escaped_name);

	free(escaped_name);

	return key;
}

struct color *config_get_sensor_color(char *sensor_name,
				      struct color *default_color)
{
	char *key = config_get_sensor_key(sensor_name);

	char *scolor = gconf_client_get_string(client,
					       key,
					       NULL);

	struct color *color = NULL;

	if (scolor)
		color = string_to_color(scolor);

	if (!scolor || !color) {
		color = color_new(default_color->red,
				  default_color->green, default_color->blue);

		scolor = color_to_string(color);

		gconf_client_set_string(client, key, scolor, NULL);
	}

	free(scolor);
	free(key);

	return color;
}

void config_set_sensor_color(char *sensor_name, struct color *color)
{
	char *key = config_get_sensor_key(sensor_name);

	char *scolor = color_to_string(color);

	gconf_client_set_string(client, key, scolor, NULL);

	free(scolor);
}

void config_init()
{
	client = gconf_client_get_default();
}

struct config *config_load()
{
	struct config *cfg = malloc(sizeof(struct config));

	cfg->graph_bgcolor = config_get_background_color();
	cfg->graph_fgcolor = config_get_foreground_color();
	cfg->graph_bg_alpha = config_get_graph_background_alpha();
	cfg->alpha_channel_enabled = config_is_alpha_channel_enabled();

	return cfg;
}
