/*
    Copyright (C) 2010 wpitchoune@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301 USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gtk/gtk.h>

#include <sensors/sensors.h>
#include <sensors/error.h>

#include "config.h"
#include "hdd.h"
#include "psensor.h"
#include "graph.h"
#include "nvidia.h"
#include "ui.h"
#include "ui_sensorlist.h"
#include "ui_color.h"

#ifdef HAVE_APP_INDICATOR
#include "ui_appindicator.h"
#endif

#ifndef PSENSOR_VERSION
#define PSENSOR_VERSION "0.0.0"
#endif

#ifndef PSENSOR_DIR
#define PSENSOR_DIR "/usr/share/psensor"
#endif

#define PSENSOR_ICON_FILENAME "psensor-32x32.png"

unsigned long debug_level = 0;

/*
  gtk_dialog_get_content_area exists since gtk 2.14
   As debian 5.0.6 (lenny) is using gtk 2.12 defines the function
   by using a macro.
*/

#if !defined(gtk_dialog_get_content_area)
#define gtk_dialog_get_content_area(dialog) GTK_DIALOG(dialog)->vbox
#endif

static void
cb_change_bgcolor(gpointer data, guint callback_action, GtkWidget * menu_item)
{
	struct ui_psensor *uipsensor = (struct ui_psensor *)data;

	if (ui_change_color("Select background color",
			    uipsensor->config->graph_bgcolor))
		config_set_background_color(uipsensor->config->graph_bgcolor);
}

static void
cb_change_fgcolor(gpointer data, guint callback_action, GtkWidget * menu_item)
{
	struct ui_psensor *uipsensor = (struct ui_psensor *)data;

	if (ui_change_color("Select foreground color",
			    uipsensor->config->graph_fgcolor))
		config_set_foreground_color(uipsensor->config->graph_fgcolor);
}

static void
cb_change_bgopacity(gpointer data, guint callback_action, GtkWidget * item)
{
	struct ui_psensor *uipsensor = (struct ui_psensor *)data;

	GtkWidget *diag
	    = gtk_dialog_new_with_buttons("Change Background Opacity",
					  uipsensor->main_window,
					  GTK_DIALOG_MODAL |
					  GTK_DIALOG_DESTROY_WITH_PARENT,
					  GTK_STOCK_OK,
					  GTK_RESPONSE_ACCEPT,
					  GTK_STOCK_CANCEL,
					  GTK_RESPONSE_REJECT,
					  NULL);

	GtkWidget *content_area = gtk_dialog_get_content_area(GTK_DIALOG(diag));

	GtkWidget *scale = gtk_hscale_new_with_range(0, 1, 0.1);

	gtk_range_set_value(GTK_RANGE(scale),
			    uipsensor->config->graph_bg_alpha);

	gtk_container_add(GTK_CONTAINER(content_area), scale);

	gtk_widget_show_all(content_area);

	gint result = gtk_dialog_run(GTK_DIALOG(diag));

	if (result == GTK_RESPONSE_ACCEPT) {
		double value = gtk_range_get_value(GTK_RANGE(scale));
		uipsensor->config->graph_bg_alpha = value;

		if (value > 0.9)
			uipsensor->config->alpha_channel_enabled = 0;
		else
			uipsensor->config->alpha_channel_enabled = 1;

		config_set_alpha_channel_enabled
		    (uipsensor->config->alpha_channel_enabled);

		config_set_graph_background_alpha(value);
	}

	gtk_widget_destroy(diag);

}

static GtkItemFactoryEntry menu_items[] = {
	{"/Foreground Color",
	 NULL, cb_change_fgcolor, 0, "<StockItem>", GTK_STOCK_SELECT_COLOR},

	{"/Background Color",
	 NULL, cb_change_bgcolor, 0, "<StockItem>", GTK_STOCK_SELECT_COLOR},

	{"/Background Opacity",
	 NULL, cb_change_bgopacity, 0, "<Item>"},

	{"/sep1",
	 NULL, NULL, 0, "<Separator>"},

	{"/Quit",
	 "", gtk_main_quit, 0, "<StockItem>", GTK_STOCK_QUIT},
};

static gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);

void update_information(struct psensor **sensors)
{
	while (1) {

		gdk_threads_enter();

#ifdef NVIDIA_SUPPORT
		nvidia_psensor_list_update(sensors);
#endif

		lmsensor_psensor_list_update(sensors);

		if (psensor_list_contains_type(sensors, SENSOR_TYPE_HDD_TEMP))
			hdd_psensor_list_update(sensors);

		gdk_threads_leave();

		sleep(1);
	}
}

gboolean refresh_thread(gpointer data)
{
	struct ui_psensor *ui_psensor = (struct ui_psensor *)data;

	gdk_threads_enter();

	update_graph(ui_psensor->sensors,
		     ui_psensor->w_graph, ui_psensor->config);

	ui_sensorlist_update(ui_psensor->ui_sensorlist);

#ifdef HAVE_APP_INDICATOR
	ui_appindicator_update(ui_psensor);
#endif

	gdk_threads_leave();

	return TRUE;
}

gboolean
on_expose_event(GtkWidget * widget, GdkEventExpose * event, gpointer data)
{
	struct ui_psensor *ui_psensor = (struct ui_psensor *)data;

	update_graph(ui_psensor->sensors,
		     ui_psensor->w_graph, ui_psensor->config);

	return FALSE;
}

void on_destroy(GtkWidget * widget, gpointer data)
{
	gtk_main_quit();
}

GdkPixbuf *create_pixbuf(const gchar * filename)
{
	GdkPixbuf *pixbuf;
	GError *error = NULL;

	pixbuf = gdk_pixbuf_new_from_file(filename, &error);

	if (pixbuf) {
		return pixbuf;
	} else {
		fprintf(stderr, "%s\n", error->message);
		g_error_free(error);
		return NULL;
	}
}

char *get_psensor_icon_path()
{
	int pathlength
	    = strlen(PSENSOR_DIR) + 1 + strlen(PSENSOR_ICON_FILENAME) + 1;
	char *path = malloc(pathlength);

	strcpy(path, PSENSOR_DIR);

	if (PSENSOR_DIR[strlen(PSENSOR_DIR) - 1] != '/')
		strcat(path, "/");

	strcat(path, PSENSOR_ICON_FILENAME);

	return path;
}

GdkPixbuf *create_psensor_icon()
{
	char *path = get_psensor_icon_path();

	GdkPixbuf *pixbuf = create_pixbuf(path);

	free(path);

	return pixbuf;
}

void associate_colors(struct psensor **sensors)
{
	// number of uniq colors
#define COLORS_COUNT 8

	unsigned int colors[COLORS_COUNT][3] = {
		{0x0000, 0x0000, 0x0000},	// black
		{0xffff, 0x0000, 0x0000},	// red
		{0x0000, 0.0000, 0xffff},	// blue
		{0x0000, 0xffff, 0x0000},	// green

		{0x7fff, 0x7fff, 0x7fff},	// grey
		{0x7fff, 0x0000, 0x0000},	// dark red
		{0x0000, 0x0000, 0x7fff},	// dark blue
		{0x0000, 0x7fff, 0x0000}	// dark green
	};

	struct psensor **sensor_cur = sensors;
	int i = 0;
	while (*sensor_cur) {
		struct color default_color;
		color_set(&default_color,
			  colors[i % COLORS_COUNT][0],
			  colors[i % COLORS_COUNT][1],
			  colors[i % COLORS_COUNT][2]);

		(*sensor_cur)->color
		    = config_get_sensor_color((*sensor_cur)->id,
					      &default_color);

		sensor_cur++;
		i++;
	}
}

GtkWidget *ui_get_popupmenu(gpointer data)
{
	GtkItemFactory *item_factory;
	GtkWidget *menu;

	/* Same as before but don't bother with the accelerators */
	item_factory = gtk_item_factory_new(GTK_TYPE_MENU, "<main>", NULL);
	gtk_item_factory_create_items(item_factory,
				      nmenu_items, menu_items, data);
	menu = gtk_item_factory_get_widget(item_factory, "<main>");

	return menu;
}

int on_graph_clicked(GtkWidget * widget, GdkEventButton * event, gpointer data)
{
	if (event->type != GDK_BUTTON_PRESS)
		return FALSE;

	GtkWidget *menu = ui_get_popupmenu(data);

	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
		       event->button, event->time);

	return TRUE;
}

GtkWidget *ui_window_create(struct ui_psensor * ui)
{
	GtkWidget *window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

	gtk_window_set_default_size(GTK_WINDOW(window), 800, 200);

	gtk_window_set_title(GTK_WINDOW(window),
			     "Psensor - Temperature Monitor");

	GdkScreen *screen = gtk_widget_get_screen(window);
	GdkColormap *colormap;

	if (ui->config->alpha_channel_enabled &&
	    gdk_screen_is_composited(screen) &&
	    (colormap = gdk_screen_get_rgba_colormap(screen)))
		gtk_widget_set_colormap(window, colormap);
	else
		ui->config->alpha_channel_enabled = 0;

	GdkPixbuf *icon = create_psensor_icon();
	if (icon)
		gtk_window_set_icon(GTK_WINDOW(window), icon);

	g_signal_connect(window, "destroy", G_CALLBACK(on_destroy), NULL);

	return window;
}

int main(int argc, char **argv)
{
	struct ui_psensor ui_psensor;

	printf("Psensor - Temperature Monitor - v%s by wpitchoune@gmail.com\n",
	       PSENSOR_VERSION);
	printf("Released under GPLv2 license terms, read LICENSE file\n");
	printf("http://wpitchoune.net/psensor\n");

	g_thread_init(NULL);
	gdk_threads_init();
	gdk_threads_enter();

	gtk_init(&argc, &argv);
	config_init();

	ui_psensor.config = config_load();

	int err = sensors_init(NULL);
	if (err) {
		fprintf(stderr, "ERROR: %s\n", sensors_strerror(err));
		return 1;
	}

	ui_psensor.sensors = get_all_sensors();
	associate_colors(ui_psensor.sensors);

	//////////// main window
	GtkWidget *window = ui_window_create(&ui_psensor);
	ui_psensor.main_window = GTK_WINDOW(window);

	//////////// main box in the window
	GtkWidget *mbox = gtk_hpaned_new();
	gtk_container_add(GTK_CONTAINER(window), mbox);

	//////////// drawing box
	GtkDrawingArea *drawing_area = GTK_DRAWING_AREA(gtk_drawing_area_new());
	ui_psensor.w_graph = GTK_WIDGET(drawing_area);

	gtk_paned_pack1(GTK_PANED(mbox), GTK_WIDGET(drawing_area), TRUE, TRUE);

	g_signal_connect(G_OBJECT(drawing_area),
			 "expose-event",
			 G_CALLBACK(on_expose_event), &ui_psensor);

	gtk_widget_add_events(GTK_WIDGET(drawing_area), GDK_BUTTON_PRESS_MASK);
	gtk_signal_connect(GTK_OBJECT(drawing_area),
			   "button_press_event",
			   (GCallback) on_graph_clicked, &ui_psensor);

	///////////// sensor list
	ui_psensor.ui_sensorlist = ui_sensorlist_create(ui_psensor.sensors);

	GtkWidget *scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	gtk_container_add(GTK_CONTAINER(scrolled_window),
			  ui_psensor.ui_sensorlist->widget);

	gtk_paned_pack2(GTK_PANED(mbox), scrolled_window, FALSE, TRUE);

	gtk_widget_show_all(window);

	GError *error;
	GThread *thread = g_thread_create((GThreadFunc) update_information,
					  ui_psensor.sensors, TRUE, &error);

	if (!thread)
		g_error_free(error);

	g_timeout_add(1000, refresh_thread, &ui_psensor);

#ifdef HAVE_APP_INDICATOR
	ui_appindicator_init(&ui_psensor);
#endif

	// main loop
	gtk_main();
	gdk_threads_leave();

	sensors_cleanup();

	psensor_list_free(ui_psensor.sensors);

	return 0;
}
