/*
    Copyright (C) 2010 wpitchoune@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301 USA 
*/

#include <stdlib.h>
#include <string.h>

#include <sensors/sensors.h>
#include <sensors/error.h>

#include "config.h"
#include "hdd.h"
#include "psensor.h"
#include "nvidia.h"

int psensor_init()
{
	int err = sensors_init(NULL);
	if (err) {
		fprintf(stderr,
			"ERROR: Sensors initialization failure: %s\n",
			sensors_strerror(err));
		return 0;
	} else {
		return 1;
	}
}

struct psensor *psensor_create(char *id, char *name, int type)
{
	struct psensor *psensor
	    = (struct psensor *)malloc(sizeof(struct psensor));

	psensor->id = id;
	psensor->name = name;
	psensor->enabled = 1;
	psensor->min = 0;
	psensor->max = 0;

	psensor->type = type;

	int t;
	for (t = 0; t < MAX_MEASURES; t++)
		psensor->values[t] = 0;

	return psensor;
}

void psensor_free(struct psensor *sensor)
{
	if (sensor) {
		free(sensor->name);
		free(sensor->id);

		if (sensor->color)
			free(sensor->color);

		free(sensor);
	}
}

void psensor_list_free(struct psensor **sensors)
{
	struct psensor **sensor_cur;

	if (sensors) {
		sensor_cur = sensors;

		while (*sensor_cur) {
			psensor_free(*sensor_cur);

			sensor_cur++;
		}

		free(sensors);
	}
}

int psensor_list_size(struct psensor **sensors)
{
	if (!sensors)
		return 0;

	int size = 0;
	struct psensor **sensor_cur = sensors;

	while (*sensor_cur) {
		size++;
		sensor_cur++;
	}
	return size;
}

int psensor_list_contains_type(struct psensor **sensors, int type)
{
	if (!sensors)
		return 0;

	struct psensor **s = sensors;
	while (*s) {
		if ((*s)->type == type)
			return 1;
		s++;
	}

	return 0;
}

struct psensor **psensor_list_add(struct psensor **sensors,
				  struct psensor *sensor)
{
	int size = psensor_list_size(sensors);

	struct psensor **result
	    = malloc((size + 1 + 1) * sizeof(struct psensor *));

	if (sensors) {
		memcpy(result, sensors, size * sizeof(struct psensor *));
	}

	result[size] = sensor;
	result[size + 1] = NULL;

	return result;
}

int is_temp_type(int type)
{
	return type & SENSOR_TYPE_TEMP;
}

int is_fan_type(int type)
{
	return type & SENSOR_TYPE_FAN;
}

char *psensor_value_to_string(int type, double value)
{
	// should not be possible to exceed 20
	// characters with temp or rpm values
	// the .x part is never displayed
	char *str = malloc(20);

	char *unit;

	if (is_temp_type(type))
		unit = "C";
	else
		unit = "";

	sprintf(str, "%.0f%s", value, unit);

	return str;
}

void psensor_set_current_value(struct psensor *sensor, double value)
{
	int i;
	for (i = 1; i < MAX_MEASURES; i++)
		sensor->values[i - 1] = sensor->values[i];

	sensor->values[MAX_MEASURES - 1] = value;

	if (!sensor->min || value < sensor->min)
		sensor->min = value;

	if (!sensor->max || value > sensor->max)
		sensor->max = value;
}

/*
  Returns the minimal value of a given 'type' (SENSOR_TYPE_TEMP or
  SENSOR_TYPE_FAN)
 */
double get_min_value(struct psensor **sensors, int type)
{
	int m = 0;
	struct psensor **s = sensors;

	while (*s) {
		struct psensor *sensor = *s;

		if (sensor->enabled && (sensor->type & type)) {
			int i;
			int t;
			for (i = 0; i < MAX_MEASURES; i++) {
				t = sensor->values[i];

				if (t && (t < m || m == 0))
					m = t;
			}
		}
		s++;
	}

	return m;

}

/*
  Returns the maximal value of a given 'type' (SENSOR_TYPE_TEMP or
  SENSOR_TYPE_FAN)
 */
double get_max_value(struct psensor **sensors, int type)
{
	int m = 0;
	struct psensor **s = sensors;

	while (*s) {
		struct psensor *sensor = *s;

		if (sensor->enabled && (sensor->type & type)) {
			int i;
			int t;
			for (i = 0; i < MAX_MEASURES; i++) {
				t = sensor->values[i];

				if (t > m)
					m = t;
			}
		}
		s++;
	}

	return m;

}

int get_min_temp(struct psensor **sensors)
{
	return get_min_value(sensors, SENSOR_TYPE_TEMP);
}

int get_min_rpm(struct psensor **sensors)
{
	return get_min_value(sensors, SENSOR_TYPE_FAN);
}

int get_max_rpm(struct psensor **sensors)
{
	return get_max_value(sensors, SENSOR_TYPE_FAN);
}

int get_max_temp(struct psensor **sensors)
{
	return get_max_value(sensors, SENSOR_TYPE_TEMP);
}

double get_value(const sensors_chip_name * name, const sensors_subfeature * sub)
{
	double val;
	int err;

	err = sensors_get_value(name, sub->number, &val);
	if (err) {
		fprintf(stderr, "ERROR: Can't get value of subfeature %s: %s\n",
			sub->name, sensors_strerror(err));
		val = 0;
	}

	return val;
}

double
get_sensor_temp_input(const sensors_chip_name * chip,
		      const sensors_feature * feature)
{
	const sensors_subfeature *sf;
	double val;

	sf = sensors_get_subfeature(chip, feature,
				    SENSORS_SUBFEATURE_TEMP_INPUT);

	if (sf) {
		val = get_value(chip, sf);
		if (val < 0.0) {
			val = 0.0;
		}
	} else {
		val = 0.0;
	}

	return val;
}

double get_fan_input(struct psensor *sensor)
{
	const sensors_chip_name *chip = sensor->iname;
	const sensors_feature *feature = sensor->feature;

	const sensors_subfeature *sf;

	sf = sensors_get_subfeature(chip,
				    feature, SENSORS_SUBFEATURE_FAN_INPUT);
	if (sf)
		return get_value(chip, sf);
	else
		return 0;
}

double get_temp_input(struct psensor *sensor)
{
	if (sensor->type == SENSOR_TYPE_LMSENSOR_TEMP)
		return get_sensor_temp_input(sensor->iname, sensor->feature);
	return 0;
}

void lmsensor_psensor_list_update(struct psensor **sensors)
{
	struct psensor **s_ptr = sensors;

	while (*s_ptr) {
		struct psensor *sensor = *s_ptr;

		if (sensor->type == SENSOR_TYPE_LMSENSOR_TEMP)
			psensor_set_current_value(sensor,
						  get_temp_input(sensor));
		else if (sensor->type == SENSOR_TYPE_LMSENSOR_FAN)
			psensor_set_current_value(sensor,
						  get_fan_input(sensor));

		s_ptr++;
	}
}

struct psensor *create_sensor(const sensors_chip_name * chip,
			      const sensors_feature * feature)
{
	char name[200];

	if (sensors_snprintf_chip_name(name, 200, chip) < 0)
		return NULL;

	sensors_subfeature_type fault_subfeature;

	if (feature->type == SENSORS_FEATURE_TEMP) {
		fault_subfeature = SENSORS_SUBFEATURE_TEMP_FAULT;

	} else if (feature->type == SENSORS_FEATURE_FAN) {
		fault_subfeature = SENSORS_SUBFEATURE_FAN_FAULT;

	} else {
		fprintf(stderr, "ERROR: create_sensor, wrong feature type\n");
		return NULL;
	}

	const sensors_subfeature *sf = sensors_get_subfeature(chip,
							      feature,
							      fault_subfeature);
	if (sf && get_value(chip, sf))
		return NULL;

	char *label;
	if (!(label = sensors_get_label(chip, feature)))
		return NULL;

	if (feature->type == SENSORS_FEATURE_TEMP
	    && (get_sensor_temp_input(chip, feature) <= 0.0))
		return NULL;

	int type = 0;
	if (feature->type == SENSORS_FEATURE_TEMP)
		type = SENSOR_TYPE_LMSENSOR_TEMP;
	else if (feature->type == SENSORS_FEATURE_FAN)
		type = SENSOR_TYPE_LMSENSOR_FAN;
	else
		return NULL;

	char *id =
	    malloc(strlen("lmsensor ") + 1 + strlen(name) + 1 + strlen(label) +
		   1);
	sprintf(id, "lmsensor %s %s", name, label);

	struct psensor *psensor = psensor_create(id, strdup(label), type);

	psensor->iname = chip;
	psensor->feature = feature;

	return psensor;
}

struct psensor **get_all_sensors()
{
	struct psensor **psensors = NULL;
	int count = 0;
	const sensors_chip_name *chip;
	int chip_nr = 0;

	while ((chip = sensors_get_detected_chips(NULL, &chip_nr))) {
		const sensors_feature *feature;
		int i = 0;
		while ((feature = sensors_get_features(chip, &i))) {
			struct psensor *psensor;

			if ((feature->type == SENSORS_FEATURE_TEMP
			     || feature->type == SENSORS_FEATURE_FAN)
			    && (psensor = create_sensor(chip, feature))) {
				struct psensor **tmp_psensors
				    = psensor_list_add(psensors,
						       psensor);

				free(psensors);

				psensors = tmp_psensors;

				count++;
			}
		}
	}

#ifdef NVIDIA_SUPPORT
	int nvidia_gpus_count = nvidia_init();
	int i;
	for (i = 0; i < nvidia_gpus_count; i++) {
		struct psensor *sensor = nvidia_create_sensor(i);

		struct psensor **tmp_psensors = psensor_list_add(psensors,
								 sensor);

		free(psensors);

		psensors = tmp_psensors;

		count++;
	}
#endif

	struct psensor **tmp_psensors = hdd_psensor_list_add(psensors);

	if (tmp_psensors != psensors) {
		free(psensors);
		psensors = tmp_psensors;
	}

	if (!psensors) {
		psensors = malloc(sizeof(struct psensor *));
		*psensors = NULL;
	}

	return psensors;
}
