/*
    Copyright (C) 2010 wpitchoune@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdlib.h>

#include "ui_sensorlist.h"
#include "config.h"
#include "ui_color.h"

enum {
	COL_NAME = 0,
	COL_TEMP,
	COL_TEMP_MIN,
	COL_TEMP_MAX,
	COL_COLOR,
	COL_COLOR_STR,
	COL_ENABLED,
	COLS_COUNT
};

void ui_sensorlist_update(struct ui_sensorlist *list)
{
	GtkTreeIter iter;
	GtkTreeModel *model
	    = gtk_tree_view_get_model(GTK_TREE_VIEW(list->widget));
	gboolean valid = gtk_tree_model_get_iter_first(model, &iter);
	struct psensor **sensor = list->sensors;

	while (valid && *sensor) {
		char *str;

		str = psensor_value_to_string((*sensor)->type,
					      (*sensor)->values[MAX_MEASURES -
								1]);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, COL_TEMP, str,
				   -1);
		free(str);

		str = psensor_value_to_string((*sensor)->type, (*sensor)->min);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter,
				   COL_TEMP_MIN, str, -1);
		free(str);

		str = psensor_value_to_string((*sensor)->type, (*sensor)->max);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter,
				   COL_TEMP_MAX, str, -1);
		free(str);

		valid = gtk_tree_model_iter_next(model, &iter);
		sensor++;
	}
}

/*
 * Returns the sensor corresponding to the x/y position
 * in the table.
 *
 * <null> if none.
 */
struct psensor *ui_sensorlist_get_sensor_at_pos(GtkTreeView * view,
						int x,
						int y, struct psensor **sensors)
{
	GtkTreePath *path;

	gtk_tree_view_get_path_at_pos(view, x, y, &path, NULL, NULL, NULL);

	if (path) {
		gint *i = gtk_tree_path_get_indices(path);
		if (i)
			return *(sensors + *i);
	}
	return NULL;
}

/*
 * Returns the index of the column corresponding
 * to the x position in the table.
 *
 * -1 if none
 */
int ui_sensorlist_get_col_index_at_pos(GtkTreeView * view, int x)
{
	GList *columns = gtk_tree_view_get_columns(view);
	GList *node;
	int colx = 0;
	int coli = 0;

	for (node = columns; node; node = node->next) {
		GtkTreeViewColumn *checkcol = (GtkTreeViewColumn *) node->data;

		if (x >= colx && x < (colx + checkcol->width))
			return coli;
		else
			colx += checkcol->width;

		coli++;
	}

	return -1;
}

void ui_sensorlist_update_sensors_color(struct ui_sensorlist *list)
{
	GtkTreeIter iter;
	GtkTreeModel *model
	    = gtk_tree_view_get_model(GTK_TREE_VIEW(list->widget));
	gboolean valid = gtk_tree_model_get_iter_first(model, &iter);
	struct psensor **sensor = list->sensors;

	while (valid && *sensor) {
		GdkColor color;
		color.red = (*sensor)->color->red;
		color.green = (*sensor)->color->green;
		color.blue = (*sensor)->color->blue;

		gchar *scolor = gdk_color_to_string(&color);

		gtk_list_store_set((GtkListStore *) model,
				   &iter, COL_COLOR_STR, scolor, -1);
		free(scolor);

		valid = gtk_tree_model_iter_next(model, &iter);
		sensor++;
	}
}

int
ui_sensorlist_on_color_clicked(GtkWidget * widget,
			       GdkEventButton * event, gpointer data)
{
	struct ui_sensorlist *list = (struct ui_sensorlist *)data;
	GtkTreeView *view = GTK_TREE_VIEW(list->widget);

	struct psensor *sensor = ui_sensorlist_get_sensor_at_pos(view,
								 event->x,
								 event->y,
								 list->sensors);

	if (sensor) {
		int coli = ui_sensorlist_get_col_index_at_pos(view,
							      event->x);

		if (coli == COL_COLOR) {
			if (ui_change_color
			    ("Select foreground color", sensor->color)) {
				ui_sensorlist_update_sensors_color(list);
				config_set_sensor_color(sensor->id,
							sensor->color);
			}
		}

	}
	return FALSE;
}

void
ui_sensorlist_on_toggled(GtkCellRendererToggle * cell,
			 gchar * path_str, gpointer data)
{
	struct ui_sensorlist *list = (struct ui_sensorlist *)data;
	GtkTreeModel *model =
	    gtk_tree_view_get_model(GTK_TREE_VIEW(list->widget));
	GtkTreeIter iter;
	GtkTreePath *path = gtk_tree_path_new_from_string(path_str);
	gboolean fixed;

	gtk_tree_model_get_iter(model, &iter, path);
	gtk_tree_model_get(model, &iter, COL_ENABLED, &fixed, -1);

	fixed ^= 1;

	gint *i = gtk_tree_path_get_indices(path);
	if (i) {
		int n = *i;
		struct psensor **sensor = list->sensors;
		while (n--)
			sensor++;
		(*sensor)->enabled = fixed;
	}

	gtk_list_store_set(GTK_LIST_STORE(model), &iter, COL_ENABLED, fixed,
			   -1);

	gtk_tree_path_free(path);
}

struct ui_sensorlist *ui_sensorlist_create(struct psensor **sensors)
{
	struct ui_sensorlist *list = malloc(sizeof(struct ui_sensorlist));

	list->sensors = sensors;

	GtkListStore *store = gtk_list_store_new(COLS_COUNT,
						 G_TYPE_STRING,
						 G_TYPE_STRING,
						 G_TYPE_STRING,
						 G_TYPE_STRING,
						 G_TYPE_STRING,
						 G_TYPE_STRING,
						 G_TYPE_BOOLEAN);

	GtkWidget *view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));

	list->widget = view;

	gtk_tree_selection_set_mode(gtk_tree_view_get_selection
				    (GTK_TREE_VIEW(view)), GTK_SELECTION_NONE);

	GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
						    -1,
						    "Sensor",
						    renderer,
						    "text", COL_NAME, NULL);

	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
						    -1,
						    "Value",
						    renderer,
						    "text", COL_TEMP, NULL);

	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
						    -1,
						    "Min",
						    renderer,
						    "text", COL_TEMP_MIN, NULL);

	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
						    -1,
						    "Max",
						    renderer,
						    "text", COL_TEMP_MAX, NULL);

	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
						    -1,
						    "Color",
						    renderer,
						    "text", COL_COLOR,
						    "background", COL_COLOR_STR,
						    NULL);

	g_signal_connect(view,
			 "button-press-event",
			 (GCallback) ui_sensorlist_on_color_clicked, list);

	renderer = gtk_cell_renderer_toggle_new();
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
						    -1,
						    "Enabled",
						    renderer,
						    "active", COL_ENABLED,
						    NULL);
	g_signal_connect(G_OBJECT(renderer), "toggled",
			 (GCallback) ui_sensorlist_on_toggled, list);

	struct psensor **sensor_cur = sensors;
	while (*sensor_cur) {
		GtkTreeIter iter;
		GdkColor color;
		color.red = (*sensor_cur)->color->red;
		color.green = (*sensor_cur)->color->green;
		color.blue = (*sensor_cur)->color->blue;

		gchar *scolor = gdk_color_to_string(&color);

		gtk_list_store_append(store, &iter);
		gtk_list_store_set(store, &iter,
				   COL_NAME, (*sensor_cur)->name,
				   COL_TEMP, "N/A",
				   COL_TEMP_MIN, "N/A",
				   COL_TEMP_MAX, "N/A",
				   COL_COLOR_STR, scolor,
				   COL_ENABLED, (*sensor_cur)->enabled, -1);

		free(scolor);

		sensor_cur++;
	}

	return list;

}
